<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\AdditionalType;
use App\Models\Category;
use Illuminate\Http\Request;

class AdditionalTypesController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function get_sub_category_ajax(Request $request)
    {
        if ($request->id == null) return response()->json(['status' => 0]);

        $sub_categories = Category::where(['parent_id' => $request->id, 'status' => 'active'])->get();
        return response()->json(['status' => 1, 'data' => $sub_categories]);
    }


    public function index()
    {
        $rows = AdditionalType::orderByDesc('id')->get();
        return view('admin.additional_types.index', compact('rows'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $categories = Category::where(['status' => 'active', 'parent_id' => null])->get();
        return view('admin.additional_types.create', compact('categories'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'name_ar' => 'required',
            'name_en' => 'required',
            'category_id' => 'required',
            'status' => 'required',
            'image' => 'image',
        ]);
        $types = new AdditionalType();
        $types['name->ar'] = $request->name_ar;
        $types['name->en'] = $request->name_en;
        if ($request->sub_sub_category != null) {
            $types['category_id'] = $request->sub_sub_category;
        } elseif ($request->sub_category != null) {
            $types['category_id'] = $request->sub_category;
        } else {
            $types['category_id'] = $request->category_id;
        }
        if ($request->image) {
            $types->setImageAttribute();
        }
        $types['status'] = $request->status;
        $types->save();

        return redirect()->route('additional_types.index')->with('message', 'تم اضافه بنجاح');
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $categories = Category::where(['status' => 'active', 'parent_id' => null])->get();

        $row = AdditionalType::find($id);
        return view('admin.additional_types.edit', compact('row', 'categories'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'name_ar' => 'required',
            'name_en' => 'required',
            'category_id' => 'required',
            'status' => 'required',
            'image' => 'image',
        ]);
        $types = AdditionalType::find($id);
        $types['name->ar'] = $request->name_ar;
        $types['name->en'] = $request->name_en;
        $types['category_id'] = $request->category_id;
        $types['status'] = $request->status;
        if ($request['image']) {
            $types->setImageAttribute();
        }
        $types->update();
        return redirect()->route('additional_types.index')->with('message', 'تم تعديل  بنجاح');

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $row = AdditionalType::find($id);
        $row->deleteImage();
        AdditionalType::destroy($id);
        return redirect()->back()->with('message', 'تم الحذف بنجاح');
    }
}
