<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Category;
use Illuminate\Http\Request;

class SubcategoriesController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index($id)
    {

    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create($id)
    {
        $category = Category::find($id);
        return view('admin.subcategories.create', compact('category'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'name_ar' => 'required',
            'name_en' => 'required',
            'image' => 'required|image',
            'parent_id' => 'required|exists:categories,id',
            'status' => 'required|in:active,not_active',
            'car_option' => 'nullable|in:true,false'
        ]);
        $category = new Category();
        $category['name->ar'] = $request['name_ar'];
        $category['name->en'] = $request['name_en'];
        $category->setImageAttribute();
        $category['parent_id'] = $request['parent_id'];
        $category['status'] = $request['status'];
        $category['car_option'] =  'false';
        $category->save();
        return redirect()->route('categories.show', $category->parent->id)->with('message', 'تم الإنشاء بنجاح');
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $row = Category::find($id);
        return view('admin.subcategories.edit', compact('row'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'name_ar' => 'required',
            'name_en' => 'required',
            'image' => 'image',
            'status' => 'required|in:active,not_active',
            'car_option' => 'nullable|in:true,false'
        ]);
        $category = Category::find($id);
        $category['name->ar'] = $request['name_ar'];
        $category['name->en'] = $request['name_en'];
        if ($request->hasFile('image')) $category->setImageAttribute();
        $category['status'] = $request['status'];
        $category['car_option'] =  'false';
        $category->update();
        return redirect()->route('categories.show', $category->parent_id)->with('message', 'تم التعديل  بنجاح');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
