<?php

namespace App\Http\Controllers\Api;

use App\Http\Resources\ProductsResource;
use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\Comment;
use App\Models\Favourite;
use App\Models\Follower;
use App\Models\Notification;
use App\Models\Product;
use App\Models\ProductImage;
use App\Models\UserToken;
use App\Models\View;
use App\User;
use Carbon\Carbon;
use Edujugon\PushNotification\PushNotification;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Lang;
use JWTAuth;
use Tymon\JWTAuth\Exceptions\JWTException;

class ProductsController extends Controller
{

    public function single_product(Request $request, $id)
    {
        if (request()->header('Authorization')) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }
            $check = View::where(['user_id' => $user->id, 'product_id' => $id])->first();
            if (!$check) {
                View::create([
                    'user_id' => $user->id,
                    'product_id' => $id
                ]);
                $product = Product::find($id);
                $product->views += 1;
                $product->update();
            }


        }

        $data = new ProductsResource(Product::find($id));
        return response()->json(['value' => 'true', 'data' => $data, 'code' => 200]);
    }

    public function my_favorite(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $data = ProductsResource::collection($user->favourite_products);
        return response()->json(['value' => 'true', 'data' => $data, 'code' => 200]);
    }

    public function products($id,Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $products = Product::whereCategoryId($id)->get();
        $data = ProductsResource::collection($products);
        return response()->json(['value' => 'true', 'data' => $data, 'code' => 200]);
    }

    public function add_favourite(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'product_id' => 'required|exists:products,id'
        ]);
        if ($validator->passes()) {
            $fav = Favourite::where(['user_id' => $user->id, 'product_id' => $request->product_id])->first();
            if ($fav) {
                $fav->delete();
                return response()->json(['value' => 'true', 'data' => \Lang::get('data.delete_favourite'), 'code' => 200]);
            } else {
                $add = new Favourite();
                $add->user_id = $user->id;
                $add->product_id = $request->product_id;
                $add->save();
                return response()->json(['value' => 'true', 'data' => \Lang::get('data.add_favourite'), 'code' => 200]);
            }
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }

    public function follow_product(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'product_id' => 'required|exists:products,id'
        ]);
        if ($validator->passes()) {
            $fav = Follower::where(['user_id' => $user->id, 'product_id' => $request->product_id])->first();
            if ($fav) {
                $fav->delete();
                return response()->json(['value' => 'true', 'data' => \Lang::get('data.unfollow'), 'code' => 200]);
            } else {
                $add = new Follower();
                $add->user_id = $user->id;
                $add->product_id = $request->product_id;
                $add->save();
                return response()->json(['value' => 'true', 'data' => \Lang::get('data.follow'), 'code' => 200]);
            }
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }

    public function add_comment(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        $validator = \Validator::make($request->all(), [
            'product_id' => 'required',
            'body' => 'required'
        ]);
        if ($validator->passes()) {

            $comment = new Comment();
            $comment->body = $request->body;
            $comment->product_id = $request->product_id;
            $comment->user_id = $user->id;
            $comment->save();


            $notification = new Notification();
            $notification['user_id'] = $comment->product->user_id;
            $notification['product_id'] = $request->product_id;
            $notification['title->ar'] = "تعليق جديد على " . $comment->product->title;
            $notification['title->en'] = "New comment on " . $comment->product->title;;
            $notification['body->ar'] = "تعليق جديد على " . $comment->product->title;
            $notification['body->en'] = "New comment on " . $comment->product->title;
            $notification['type'] = 'comment';
            $notification->save();
            $product = Product::find($request['product_id']);


            $push = new PushNotification('fcm');

            $all_tokens = UserToken::where(['status' => 'connected', 'user_id' => $comment->product->user_id])->get();

            foreach ($all_tokens as $token) {

                if ($token->device_type == 'ios') {
                    $msg = [
                        'data' => [
                            'title_ar' => $notification->title['ar'],
                            'title_en' => $notification->title['en'],
                            'body_ar' => $notification->body['ar'],
                            'body_en' => $notification->body['en'],
                            'type' => 'comment',
                            'product_id' => $notification->product_id,
                            'product_model' => new ProductsResource($product),
                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'notification' => [
                            'title_ar' => $notification->title['ar'],
                            'title_en' => $notification->title['en'],
                            'body_ar' => $notification->body['ar'],
                            'body_en' => $notification->body['en'],
                            'type' => 'comment',
                            'product_id' => $notification->product_id,
                            'product_model' => new ProductsResource($product),

                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();

                } else {
                    $msg = [
                        'data' => [
                            'title_ar' => $notification->title['ar'],
                            'title_en' => $notification->title['en'],
                            'body_ar' => $notification->body['ar'],
                            'body_en' => $notification->body['en'],
                            'type' => 'comment',
                            'product_id' => $notification->product_id,
                            'product_model' => new ProductsResource($product),

                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();

                }


            }


            $followed_users = $product->followed_users->pluck('id');


            foreach ($followed_users as $followed_user) {
                $notification = new Notification();
                $notification['user_id'] = $followed_user;
                $notification['product_id'] = $request->product_id;
                $notification['title->ar'] = "تعليق جديد على " . $comment->product->title;
                $notification['title->en'] = "New comment on " . $comment->product->title;;
                $notification['body->ar'] = "تعليق جديد على " . $comment->product->title;
                $notification['body->en'] = "New comment on " . $comment->product->title;
                $notification['type'] = 'comment';
                $notification->save();
            }

            $followed_users_tokens = UserToken::whereIn('user_id', $followed_users)->whereStatus('connected')->get();

            foreach ($followed_users_tokens as $new_token) {


                if ($new_token->device_type == 'ios') {
                    $msg = [
                        'data' => [
                            'title_ar' => $notification->title['ar'],
                            'title_en' => $notification->title['en'],
                            'body_ar' => $notification->body['ar'],
                            'body_en' => $notification->body['en'],
                            'type' => 'comment',
                            'product_id' => $notification->product_id,
                            'product_model' => new ProductsResource($product),

                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'notification' => [
                            'title_ar' => $notification->title['ar'],
                            'title_en' => $notification->title['en'],
                            'body_ar' => $notification->body['ar'],
                            'body_en' => $notification->body['en'],
                            'type' => 'comment',
                            'product_id' => $notification->product_id,
                            'product_model' => new ProductsResource($product),

                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($new_token->token)
                        ->send();

                } else {
                    $msg = [
                        'data' => [
                            'title_ar' => $notification->title['ar'],
                            'title_en' => $notification->title['en'],
                            'body_ar' => $notification->body['ar'],
                            'body_en' => $notification->body['en'],
                            'type' => 'comment',
                            'product_id' => $notification->product_id,
                            'product_model' => new ProductsResource($product),
                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($new_token->token)
                        ->send();

                }


            }

            $data = new ProductsResource($product);


            return response()->json(['value' => 'true', 'data' => $data, 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }


    public function store(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        if ($user->vip == 'false') {
            $limit = 2;
            $last_products = Product::where(['status' => 'active', 'user_id' => $user->id])->orderByDesc('id')->take($limit)->pluck('created_at');
        } else {
            $limit = 4;
            $last_products = Product::where(['status' => 'active', 'user_id' => $user->id])->orderByDesc('id')->take($limit)->pluck('created_at');
        }
        $count_data = 0;
        foreach ($last_products as $last_product) {
            if (Carbon::now()->toDateString() == Carbon::parse($last_product)->toDateString()) {
                $count_data += 1;
            }
        }
        if ($count_data >= $limit) {
            return response()->json(['value' => false, 'msg' => Lang::get('data.check_add')]);
        }

        if ($request['grand_id']) {
            $category = Category::find($request['grand_id']);
            if (!$category) return response()->json(['value' => false, 'msg' => 'wrong id for category']);
            if ($category->car_option == 'true') {

                $validator = \Validator::make($request->all(), [
                    'title' => 'required',
                    'grand_id' => 'required',
                    'parent_id' => 'nullable',
                    'child_id' => 'nullable',
                    'city_id' => 'required|exists:cities,id',
                    'hide_mobile' => 'in:true,false',
                    'mobile' => 'required_if:hide_mobile,false',
                    'ad_type' => 'required|in:buy,waiver',
                    'year' => 'required',
                    'car_status' => 'required|in:used,new,accident',
                    'fetis' => 'required|in:manual,automatic',
                    'fuel' => 'required|in:gasoline,hybrid,diesel',
                    'double' => 'required|in:true,false',
                    'km' => 'required',
                    'detail' => 'required',
                    'lat' => 'required',
                    'lng' => 'required',
                    'installment_destination' => 'nullable',
                    'determine_price' => 'required|in:true,false',
                    'price' => 'required_if:determine_price,false',


                ]);


            } else {
                $validator = \Validator::make($request->all(), [
                    'title' => 'required',
                    'grand_id' => 'required',
                    'parent_id' => 'nullable',
                    'child_id' => 'nullable',
                    'city_id' => 'required|exists:cities,id',
                    'hide_mobile' => 'in:true,false',
                    'mobile' => 'required_if:hide_mobile,false',
                    'ad_type' => 'in:buy,waiver',
                    'year' => 'nullable',
                    'car_status' => 'nullable|in:used,new,accident',
                    'fetis' => 'nullable|in:manual,automatic',
                    'fuel' => 'nullable|in:gasoline,hybrid,diesel',
                    'double' => 'nullable|in:true,false',
                    'km' => 'nullable',
                    'detail' => 'required',
                    'lat' => 'required',
                    'lng' => 'required',
                    'installment_destination' => 'nullable',
                    'determine_price' => 'required|in:true,false',
                    'price' => 'required_if:determine_price,false',


                ]);

            }

            if ($validator->passes()) {
                $product = new Product();
                $product['user_id'] = $user->id;
                $product['title'] = $request['title'];
                if ($request->child_id != 0) {
                    $product['category_id'] = $request['child_id'];
                } elseif ($request->grand_id != 0 && $request->parent_id != 0 && $request->child_id == 0) {
                    $product['category_id'] = $request['parent_id'];
                } elseif ($request->grand_id != 0 && $request->parent_id == 0 && $request->child_id == 0) {
                    $product['category_id'] = $request['grand_id'];
                }
                $product['city_id'] = $request['city_id'];
                $product['mobile'] = $request['mobile'];
                $product['hide_mobile'] = $request['hide_mobile'];
                $product['ad_type'] = $request['ad_type'];
                $product['year'] = $request['year'];
                $product['car_status'] = $request['car_status'];
                $product['installment_destination'] = $request['installment_destination'];
                $product['fetis'] = $request['fetis'];
                $product['fuel'] = $request['fuel'];
                $product['double'] = $request['double'];
                $product['km'] = $request['km'];
                $product['determine_price'] = $request['determine_price'];
                $product['price'] = $request['price'];
                $product['price'] = $request['price'];
                $product['detail'] = $request['detail'];
                $product['lat'] = $request['lat'];
                $product['lng'] = $request['lng'];
                $product['views'] = 0;
                $product['status'] = 'active';
                $product['code'] = uniqid();
                $product->save();

                if ($request->images) {
                    $image_array = $request['images'];
                    $i = 1;
                    foreach ($image_array as $image) {
                        $filename = time() . $i . '.' . $image->getClientOriginalExtension();
                        $image->move(public_path('images/products'), $filename);
                        ProductImage::create([
                            'product_id' => $product->id,
                            'image' => $filename
                        ]);
                        $i++;
                    }
                }

                $data = new ProductsResource($product);
                if (count($product->images) == 0) {

                    $notification = new Notification();
                    $notification['user_id'] = $user->id;
                    $notification['body->ar'] = 'عميلنا العزيز اضافه صوره لأعلانكم يزيد من نسبه التفاعل ';
                    $notification['body->en'] = 'Our dear customer, adding a picture to your advertisement increases the rate of interaction';
                    $notification['title->en'] = 'Add Images';
                    $notification['product_id'] = $product->id;
                    $notification['title->ar'] = 'إضافة صور';
                    $notification['type'] = 'admin';
                    $notification->save();

                    $all_tokens = UserToken::whereStatus('connected')->whereUserId($user->id)->get();
                    $push = new PushNotification('fcm');

                    foreach ($all_tokens as $token) {

                        if ($token->device_type == 'ios') {
                            $msg = [
                                'data' => [
                                    'title_ar' => $notification->title['ar'],
                                    'title_en' => $notification->title['en'],
                                    'body_ar' => $notification->body['ar'],
                                    'body_en' => $notification->body['en'],
                                    'type' => $notification->type,
                                    'product_id' => $product->id,
                                    'product_model' => new ProductsResource($product),

                                    'sound' => 'default',
                                    'click_action' => 'FCM_PLUGIN_ACTIVITY',
                                ],
                                'notification' => [
                                    'title_ar' => $notification->title['ar'],
                                    'title_en' => $notification->title['en'],
                                    'body_ar' => $notification->body['ar'],
                                    'body_en' => $notification->body['en'],
                                    'type' => $notification->type,
                                    'product_id' => $product->id,
                                    'product_model' => new ProductsResource($product),

                                    'sound' => 'default',
                                    'click_action' => 'FCM_PLUGIN_ACTIVITY',
                                ],
                                'priority' => 'high',
                            ];
                            $push->setMessage($msg)
                                ->setDevicesToken($token->token)
                                ->send();

                        } else {
                            $msg = [
                                'data' => [
                                    'title_ar' => $notification->title['ar'],
                                    'title_en' => $notification->title['en'],
                                    'body_ar' => $notification->body['ar'],
                                    'body_en' => $notification->body['en'],
                                    'type' => $notification->type,
                                    'product_id' => $product->id,
                                    'product_model' => new ProductsResource($product),

                                    'sound' => 'default',
                                    'click_action' => 'FCM_PLUGIN_ACTIVITY',
                                ],
                                'priority' => 'high',
                            ];
                            $push->setMessage($msg)
                                ->setDevicesToken($token->token)
                                ->send();

                        }


                    }


                }


                $user_name = $user->name ? $user->name : $user->mobile;
                $followers = $user->followers->pluck('id');
                $followers_tokens = UserToken::whereIn('user_id', $followers)->whereStatus('connected')->get();
                $push = new PushNotification('fcm');

                foreach ($followers_tokens as $token) {

                    if ($token->device_type == 'ios') {
                        $msg = [
                            'data' => [
                                'title_ar' => " قام  $user_name بإنشاء إعلان جديد  ",
                                'title_en' => "Your Following $user_name posted a new product",
                                'body_ar' => " قام  $user_name بإنشاء إعلان جديد  ",
                                'body_en' => "Your Following $user_name posted a new product",
                                'type' => "follower",
                                'product_id' => $product->id,
                                'product_model' => new ProductsResource($product),

                                'sound' => 'default',
                                'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            ],
                            'notification' => [
                                'title_ar' => " قام  $user_name بإنشاء إعلان جديد  ",
                                'title_en' => "Your Following $user_name posted a new product",
                                'body_ar' => " قام  $user_name بإنشاء إعلان جديد  ",
                                'body_en' => "Your Following $user_name posted a new product",
                                'type' => "follower",
                                'product_id' => $product->id,
                                'product_model' => new ProductsResource($product),

                                'sound' => 'default',
                                'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            ],
                            'priority' => 'high',
                        ];
                        $push->setMessage($msg)
                            ->setDevicesToken($token->token)
                            ->send();

                    } else {
                        $msg = [
                            'data' => [
                                'title_ar' => " قام  $user_name بإنشاء إعلان جديد  ",
                                'title_en' => "Your Following $user_name posted a new product",
                                'body_ar' => " قام  $user_name بإنشاء إعلان جديد  ",
                                'body_en' => "Your Following $user_name posted a new product",
                                'type' => "follower",
                                'product_id' => $product->id,
                                'product_model' => new ProductsResource($product),

                                'sound' => 'default',
                                'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            ],
                            'priority' => 'high',
                        ];
                        $push->setMessage($msg)
                            ->setDevicesToken($token->token)
                            ->send();

                    }

                    $notification = new Notification();
                    $notification['user_id'] = $token->user_id;
                    $notification['body->ar'] = " قام  $user_name بإنشاء إعلان جديد  ";
                    $notification['body->en'] = "Your Following $user_name posted a new product";
                    $notification['title->en'] = "Your Following $user_name posted a new product";
                    $notification['product_id'] = $product->id;
                    $notification['title->ar'] = " قام  $user_name بإنشاء إعلان جديد  ";
                    $notification['type'] = 'follower';
                    $notification->save();

                }


                return response()->json(['value' => true, 'data' => $data, 'msg' => \Lang::get('data.product_added'), 'code' => 200]);

            }
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);


        } else {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.category_missed'), 'code' => 422]);
        }
    }

    public function check_add_products(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $products = Product::where('user_id', $user->id)->where('created_at', '>=', Carbon::now()->subDay())->get();
        if ($user->vip == 'true' && count($products) < 4) {
            return response()->json(['value' => true, 'data' => \Lang::get('data.can_add'), 'code' => 200]);
        } elseif ($user->vip == 'false' && count($products) < 1) {
            return response()->json(['value' => true, 'data' => \Lang::get('data.can_add'), 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.cant_add'), 'code' => 422]);
        }
    }

    public function delete_product($id, Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $product = Product::find($id);
        if ($product->user_id == $user->id) {
            Product::destroy($id);
            return response()->json(['value' => true, 'data' => \Lang::get('data.deleted'), 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.not_owner'), 'code' => 401]);

        }
    }

    public function delete_comment($id, Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $comment = Comment::find($id);
        $product = Product::find($comment->product_id);

        if ($comment->user_id == $user->id || $product->user_id == $user->id) {
            Comment::destroy($id);
            return response()->json(['value' => true, 'data' => \Lang::get('data.deleted'), 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.not_allowed'), 'code' => 401]);
        }
    }

    public function stop_comment($id, Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $product = Product::find($id);
        if (!$product) return response()->json(['value' => false, 'msg' => 'wrong id', 'code' => 422]);

        if ($product->user_id == $user->id) {
            $product->stop_comments = 'close';
            $product->update();
            return response()->json(['value' => true, 'data' => \Lang::get('data.comments_stopped'), 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.not_allowed'), 'code' => 422]);
        }
    }

    public function my_product(Request $request)
    {

        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        $data = ProductsResource::collection($user->product()->where('status', 'active')->get());

        return response()->json(['value' => 'true', 'data' => $data, 'code' => 200]);
    }

    public function similar_product(Request $request, $id_category, $id_product)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $products = Product::where(['status' => 'active', 'category_id' => $id_category])->Where('id', '!=', $id_product)->orderByDesc('created_at')->take(3)->get();

        $data = ProductsResource::collection($products);

        return response()->json(['value' => 'true', 'data' => $data, 'code' => 200]);
    }

    public function search(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'search' => 'required'
        ]);
        if ($validator->passes()) {
            $product = Product::where('title', 'like', "%$request->search%")->where('status', 'active')->OrderByDesc('created_at')->paginate(10);
            return ProductsResource::collection($product)->additional(['value' => true, 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }

    public function category_products($id, Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if ($request->header('Authorization')) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }
        }

        $categories = Category::whereParentId($id)->pluck('id');
        $all_categories = $categories->merge($id);
        $products = Product::whereIn('category_id', $all_categories)->whereStatus('active')->whereHas('images')->orderByDesc('created_at')->paginate(10);

        return ProductsResource::collection($products)->additional(['value' => true, 'code' => 200]);


    }

    public function home_product(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if ($request->header('Authorization')) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }
        }
        $grand_id = $request['grand_id'];
        $parent_id = $request['parent_id'];
        $children_id = $request['children_id'];
        $lat = $request['lat'];
        $lng = $request['lng'];
        $max_distance = 30;
        $lat_small = $lat - ($max_distance * 0.018);
        $lat_large = $lat + ($max_distance * 0.018);
        $lng_small = $lng - ($max_distance * 0.018);
        $lng_large = $lng + ($max_distance * 0.018);


        if ($grand_id == 0 && $parent_id == 0 && $children_id == 0) {


            if ($request['has_image']) {
                $products = Product::where('status', 'active')->whereHas('images')->orderByDesc('created_at')->paginate(10);

            } elseif ($request['city_id']) {
                $products = Product::where('status', 'active')->where('city_id', $request['city_id'])->orderByDesc('created_at')->paginate(10);

            } elseif ($request['city_id'] && $request['has_image']) {
                $products = Product::where('status', 'active')->where('city_id', $request['city_id'])->whereHas('images')->orderByDesc('created_at')->paginate(10);

            } elseif ($request['lat']) {
                $products = Product::where('status', 'active')->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->orderByDesc('created_at')->paginate(10);

            } elseif ($request['lat'] && $request['has_image']) {
                $products = Product::where('status', 'active')->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->whereHas('images')->orderByDesc('created_at')->paginate(10);

            } elseif ($request['lat'] && $request['city_id']) {
                $products = Product::where('status', 'active')->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->where('city_id', $request['city_id'])->orderByDesc('created_at')->paginate(10);

            } elseif ($request['lat'] && $request['city_id'] && $request['has_image']) {
                $products = Product::where('status', 'active')->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->where('city_id', $request['city_id'])->whereHas('images')->orderByDesc('created_at')->paginate(10);

            } else {
                $products = Product::where('status', 'active')->orderByDesc('created_at')->get();

            }
        } elseif ($grand_id != 0 && $parent_id == 0 && $children_id == 0) {
            $parent = Category::find($grand_id);
            $childrens = Category::whereIn('parent_id', $parent->categories->pluck('id'))->pluck('id');
            $parent_categories = $parent->categories->pluck('id');
            $all_categories = $childrens->merge($parent_categories);


            if ($request['has_image']) {
                $products = Product::whereIn('category_id', $all_categories)->whereStatus('active')->whereHas('images')->orderByDesc('created_at')->paginate(10);

            } elseif ($request['city_id']) {
                $products = Product::whereIn('category_id', $all_categories)->whereStatus('active')->where('city_id', $request['city_id'])->orderByDesc('created_at')->paginate(10);

            } elseif ($request['city_id'] && $request['has_image']) {
                $products = Product::whereIn('category_id', $all_categories)->whereStatus('active')->whereHas('images')->where('city_id', $request['city_id'])->orderByDesc('created_at')->paginate(10);

            } elseif ($request['lat']) {
                $products = Product::whereIn('category_id', $all_categories)->whereStatus('active')->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->orderByDesc('created_at')->paginate(10);

            } elseif ($request['lat'] && $request['has_image']) {
                $products = Product::whereIn('category_id', $all_categories)->whereStatus('active')->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->whereHas('images')->orderByDesc('created_at')->paginate(10);

            } elseif ($request['lat'] && $request['city_id']) {
                $products = Product::whereIn('category_id', $all_categories)->whereStatus('active')->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->where('city_id', $request['city_id'])->orderByDesc('created_at')->paginate(10);

            } elseif ($request['lat'] && $request['city_id'] && $request['has_image']) {
                $products = Product::whereIn('category_id', $all_categories)->whereStatus('active')->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->where('city_id', $request['city_id'])->whereHas('images')->orderByDesc('created_at')->paginate(10);

            } else {
                $products = Product::whereIn('category_id', $all_categories)->whereStatus('active')->orderByDesc('created_at')->paginate(10);

            }


        } elseif ($grand_id != 0 && $parent_id != 0 && $children_id == 0) {

            $categories_children = Category::where(['status' => 'active', 'parent_id' => $parent_id])->pluck('id')->toArray(); // category child

            array_push($categories_children, (integer)$parent_id);

            if ($request['has_image']) {
                $products = Product::whereStatus('active')->whereIn('category_id', $categories_children)->whereHas('images')->orderByDesc('created_at')->paginate(10);

            } elseif ($request['city_id']) {
                $products = Product::whereStatus('active')->whereIn('category_id', $categories_children)->where('city_id', $request['city_id'])->orderByDesc('created_at')->paginate(10);

            } elseif ($request['city_id'] && $request['has_image']) {
                $products = Product::whereStatus('active')->whereIn('category_id', $categories_children)->whereHas('images')->where('city_id', $request['city_id'])->orderByDesc('created_at')->paginate(10);

            } elseif ($request['lat']) {
                $products = Product::whereStatus('active')->whereIn('category_id', $categories_children)->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->orderByDesc('created_at')->paginate(10);

            } elseif ($request['lat'] && $request['city_id']) {
                $products = Product::whereStatus('active')->whereIn('category_id', $categories_children)->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->where('city_id', $request['city_id'])->orderByDesc('created_at')->paginate(10);

            } elseif ($request['lat'] && $request['has_image']) {
                $products = Product::whereStatus('active')->whereIn('category_id', $categories_children)->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->whereHas('images')->orderByDesc('created_at')->paginate(10);

            } elseif ($request['lat'] && $request['has_image'] && $request['city_id']) {
                $products = Product::whereStatus('active')->whereIn('category_id', $categories_children)->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->whereHas('images')->where('city_id', $request['city_id'])->orderByDesc('created_at')->paginate(10);

            } else {
                $products = Product::whereStatus('active')->whereIn('category_id', $categories_children)->orderByDesc('created_at')->paginate(10);

            }


        } elseif ($grand_id != 0 && $parent_id != 0 && $children_id != 0) {

            $category = Category::find($children_id)->value('car_option');
            if ($category == 'true' && $request['year']) {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->where('year', $request['year'])->orderByDesc('created_at')->paginate(10);

            } elseif ($request['has_image']) {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->whereHas('images')->orderByDesc('created_at')->paginate(10);
            } elseif ($request['has_image'] && $request['year'] && $category == 'true') {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->where('year', $request['year'])->whereHas('images')->orderByDesc('created_at')->paginate(10);

            } elseif ($request['city_id']) {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->where('city_id', $request['city_id'])->orderByDesc('created_at')->paginate(10);
            } elseif ($request['city_id'] && $request['has_image']) {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->where('city_id', $request['city_id'])->whereHas('images')->orderByDesc('created_at')->paginate(10);
            } elseif ($request['city_id'] && $request['year'] && $category == 'true') {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->where('city_id', $request['city_id'])->where('year', $request['year'])->orderByDesc('created_at')->paginate(10);
            } elseif ($request['city_id'] && $request['year'] && $request['has_image'] && $category == 'true') {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->where('city_id', $request['city_id'])->whereHas('images')->where('year', $request['year'])->orderByDesc('created_at')->paginate(10);
            } elseif ($request['lat']) {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->orderByDesc('created_at')->paginate(10);
            } elseif ($request['lat'] && $request['has_image']) {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->whereHas('images')->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->orderByDesc('created_at')->paginate(10);
            } elseif ($request['lat'] && $request['city_id']) {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->where('city_id', $request['city_id'])->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->orderByDesc('created_at')->paginate(10);
            } elseif ($request['lat'] && $request['year']) {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->whereYear($request['year'])->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->orderByDesc('created_at')->paginate(10);
            } elseif ($request['lat'] && $request['has_image'] && $request['city_id']) {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->whereHas('images')->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->where('city_id', $request['city_id'])->orderByDesc('created_at')->paginate(10);
            } elseif ($request['lat'] && $request['has_image'] && $request['year']) {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->whereHas('images')->whereYear($request['year'])->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->orderByDesc('created_at')->paginate(10);
            } elseif ($request['lat'] && $request['year'] && $request['city_id']) {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->whereHas('images')->whereCityId($request['city_id'])->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->orderByDesc('created_at')->paginate(10);
            } elseif ($request['lat'] && $request['city_id'] && $request['year'] && $request['has_image']) {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->whereHas('images')->whereCityId($request['city_id'])->whereYear($request['year'])->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])->orderByDesc('created_at')->paginate(10);

            } else {
                $products = Product::where(['status' => 'active', 'category_id' => $children_id])->orderByDesc('created_at')->paginate(10);

            }

        }

        return ProductsResource::collection($products)->additional(['value' => true, 'code' => 200]);

    }

    public function latest_products(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $rows = Product::orderByDesc('created_at')->get();
        $data = ProductsResource::collection($rows);
        return response()->json(['value' => 'true', 'data' => $data, 'code' => 200]);
    }

    public function refresh_product($id, Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $product = Product::find($id);
        if ($product->updates_number < 2 && Carbon::now()->toDateString() == Carbon::parse($product->created_at)->toDateString()) {

            $product->created_at = Carbon::now();
            $product->updated_at = Carbon::now();
            $product->updates_number += 1;
            $product->update();
            return response()->json(['value' => true, 'data' => new ProductsResource($product), 'code' => 200]);

        } elseif (Carbon::now()->toDateString() != Carbon::parse($product->created_at)->toDateString()) {
            $product->created_at = Carbon::now();
            $product->updated_at = Carbon::now();
            $product->updates_number = 1;
            $product->update();
            return response()->json(['value' => true, 'data' => new ProductsResource($product), 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.cant_refresh'), 'code' => 422]);
        }
    }


    public function special_products(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $data = Product::where('status', 'active')->whereHas('user', function ($q) {
            $q->where(['status' => 'active', 'vip' => 'true']);
        })->paginate(10);
        return ProductsResource::collection($data)->additional(['value' => true, 'code' => 200]);
    }


    public function check_add_product(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        if ($user->vip == 'false') {
            $limit = 2;
            $last_products = Product::where(['status' => 'active', 'user_id' => $user->id])->orderByDesc('id')->take($limit)->pluck('created_at');
        } else {
            $limit = 4;
            $last_products = Product::where(['status' => 'active', 'user_id' => $user->id])->orderByDesc('id')->take($limit)->pluck('created_at');
        }
        $count_data = 0;
        foreach ($last_products as $last_product) {
            if (Carbon::now()->toDateString() == Carbon::parse($last_product)->toDateString()) {
                $count_data += 1;
            }
        }
        if ($count_data >= $limit) {
            return response()->json(['value' => false, 'msg' => 'لا يمكن اضافة اكثر من اعلانين فى اليوم']);
        } else {
            return response()->json(['value' => true, 'msg' => 'لم يصل الى الحد الاقصى فى اضافه الاعلانات فى اليوم']);
        }

    }
}
