<?php

namespace App\Http\Resources;

use App\Models\Follower;
use Carbon\Carbon;
use Illuminate\Http\Resources\Json\JsonResource;
use Illuminate\Support\Facades\App;
use JWTAuth;

class ProfileResource extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @param \Illuminate\Http\Request $request
     * @return array
     */
    public function toArray($request)
    {
        $lang = $request->header('Accept-Language','ar');
        \App::setLocale($lang);
        if ($request->header('Authorization')){
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }
            $check = Follower::where(['user_id'=>$user->id,'follower_id'=>$this->id])->first();

        }

        return [
            'id' => $this->id,
            'name' => $this->name ? $this->name : '',
            'email' => $this->email ? $this->email : '',
            'image' => $this->image ? asset('images/users/' . $this->image) : '',
            'mobile' => $this->mobile ? $this->mobile : '',
            'online' => $this->online,
            'status' => $this->status,
            'verified' => $this->verified ? $this->verified : '',
            'created_at' =>Carbon::parse($this->created_at)->diffForHumans(),
            'lat' => $this->lat ? $this->lat : '',
            'lng' => $this->lng ? $this->lng : '',
            'vip' => $this->vip ? $this->vip : '',
            'follow_status'=>isset($check) ? 'true' : 'false',
            'total_rates' => $this->total_rates,
            'city_id' => $this->city_id ? (integer)$this->city_id : 0,
            'country_code_id' => $this->country_code_id ? (integer)$this->country_code_id : 0,
            'products' => ProductsResource::collection($this->product),
        ];
    }
}
